/*!
  * vue-timer-hook v0.0.1
  * (c) 2021 Martin Donadieu
  * @license MIT
  */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vue = require('vue');

class Time {
    static getTimeFromSeconds(secs) {
        const totalSeconds = Math.ceil(secs);
        const days = Math.floor(totalSeconds / (60 * 60 * 24));
        const hours = Math.floor((totalSeconds % (60 * 60 * 24)) / (60 * 60));
        const minutes = Math.floor((totalSeconds % (60 * 60)) / 60);
        const seconds = Math.floor(totalSeconds % 60);
        return {
            seconds,
            minutes,
            hours,
            days,
        };
    }
    static getSecondsFromExpiry(expiry, shouldRound) {
        const now = new Date().getTime();
        const milliSecondsDistance = expiry - now;
        if (milliSecondsDistance > 0) {
            const val = milliSecondsDistance / 1000;
            return shouldRound ? Math.round(val) : val;
        }
        return 0;
    }
    static getSecondsFromPrevTime(prevTime, shouldRound) {
        const now = new Date().getTime();
        const milliSecondsDistance = now - prevTime;
        if (milliSecondsDistance > 0) {
            const val = milliSecondsDistance / 1000;
            return shouldRound ? Math.round(val) : val;
        }
        return 0;
    }
    static getSecondsFromTimeNow() {
        const now = new Date();
        const currentTimestamp = now.getTime();
        const offset = now.getTimezoneOffset() * 60;
        return currentTimestamp / 1000 - offset;
    }
    static getFormattedTimeFromSeconds(totalSeconds, format) {
        const { seconds: secondsValue, minutes, hours, } = Time.getTimeFromSeconds(totalSeconds);
        let ampm = '';
        let hoursValue = hours;
        if (format === '12-hour') {
            ampm = hours >= 12 ? 'pm' : 'am';
            hoursValue = hours % 12;
        }
        return {
            seconds: secondsValue,
            minutes,
            hours: hoursValue,
            ampm,
        };
    }
}

class Validate {
    static expiryTimestamp(expiryTimestamp) {
        const isValid = new Date(expiryTimestamp).getTime() > 0;
        if (!isValid) {
            console.warn('vue-timer-hook: { useTimer } Invalid expiryTimestamp settings', expiryTimestamp); // eslint-disable-line
        }
        return isValid;
    }
    static onExpire(onExpire) {
        const isValid = onExpire && typeof onExpire === 'function';
        if (onExpire && !isValid) {
            console.warn('vue-timer-hook: { useTimer } Invalid onExpire settings function', onExpire); // eslint-disable-line
        }
        return isValid;
    }
}

const isNumber = (val) => typeof val === 'number';
// export interface UseIntervalReturnMs {
//   start(): number;
// }
// export interface UseIntervalReturnArgs<TArgs extends Array<any>> {
//   start(_: undefined, ...args: TArgs): number;
// }
function useInterval(callback, ms) {
    let intervalId = undefined;
    const start = (_ms, ..._args) => {
        remove();
        if (!_ms && !ms) {
            return;
        }
        const m = (_ms || ms);
        return (intervalId = setInterval(callback, m
        // ...(_args && _args.length ? _args : args)
        ));
    };
    const remove = () => {
        if (!intervalId)
            return;
        clearInterval(intervalId);
        intervalId = undefined;
    };
    if (isNumber(ms)) {
        start();
    }
    vue.onUnmounted(remove);
    return { remove, start };
}

// import { useState } from 'react';
const DEFAULT_DELAY = 1000;
function getDelayFromExpiryTimestamp(expiryTimestamp) {
    if (!Validate.expiryTimestamp(expiryTimestamp)) {
        return null;
    }
    const seconds = Time.getSecondsFromExpiry(expiryTimestamp);
    const extraMilliSeconds = Math.floor((seconds - Math.floor(seconds)) * 1000);
    return extraMilliSeconds > 0 ? extraMilliSeconds : DEFAULT_DELAY;
}
function useTimer({ expiryTimestamp: expiry, autoStart = true, }) {
    const expiryTimestamp = vue.ref(expiry);
    const seconds = vue.ref(Time.getSecondsFromExpiry(expiryTimestamp.value));
    const isRunning = vue.ref(autoStart);
    const isExpired = vue.ref(false);
    const didStart = vue.ref(autoStart);
    const delay = vue.ref(getDelayFromExpiryTimestamp(expiryTimestamp.value));
    function handleExpire() {
        isExpired.value = true;
        isRunning.value = false;
        delay.value = null;
    }
    function pause() {
        isRunning.value = false;
    }
    function restart(newExpiryTimestamp, newAutoStart = true) {
        delay.value = getDelayFromExpiryTimestamp(newExpiryTimestamp);
        didStart.value = newAutoStart;
        isExpired.value = false;
        isRunning.value = newAutoStart;
        expiryTimestamp.value = newExpiryTimestamp;
        seconds.value = Time.getSecondsFromExpiry(newExpiryTimestamp);
    }
    function resume() {
        const time = new Date();
        time.setMilliseconds(time.getMilliseconds() + seconds.value * 1000);
        restart(time.getTime());
    }
    function start() {
        if (didStart) {
            seconds.value = Time.getSecondsFromExpiry(expiryTimestamp.value);
            isRunning.value = true;
        }
        else {
            resume();
        }
    }
    useInterval(() => {
        if (delay.value !== DEFAULT_DELAY) {
            delay.value = DEFAULT_DELAY;
        }
        const secondsValue = Time.getSecondsFromExpiry(expiryTimestamp.value);
        seconds.value = secondsValue;
        if (secondsValue <= 0) {
            handleExpire();
        }
    }, isRunning.value ? delay.value : null);
    return {
        ...Time.getTimeFromSeconds(seconds.value),
        start,
        pause,
        resume,
        restart,
        isRunning,
        isExpired,
    };
}

const epochSeconds = () => new Date().getTime();
function useStopwatch({ autoStart, offsetTimestamp, }) {
    const passedSeconds = vue.ref(Time.getSecondsFromExpiry(offsetTimestamp, true) || 0);
    const prevTime = vue.ref(epochSeconds());
    const seconds = vue.ref(passedSeconds.value + Time.getSecondsFromPrevTime(prevTime.value || 0, true));
    const isRunning = vue.ref(autoStart);
    useInterval(() => {
        seconds.value =
            passedSeconds.value + Time.getSecondsFromPrevTime(prevTime.value, true);
    }, isRunning.value ? 1000 : false);
    function start() {
        const newPrevTime = epochSeconds();
        prevTime.value = newPrevTime;
        isRunning.value = true;
        seconds.value =
            passedSeconds.value + Time.getSecondsFromPrevTime(newPrevTime, true);
    }
    function pause() {
        passedSeconds.value = seconds.value;
        isRunning.value = false;
    }
    function reset(offset = 0, newAutoStart = true) {
        const newPassedSeconds = Time.getSecondsFromExpiry(offset, true) || 0;
        const newPrevTime = epochSeconds();
        prevTime.value = newPrevTime;
        passedSeconds.value = newPassedSeconds;
        isRunning.value = newAutoStart;
        seconds.value =
            newPassedSeconds + Time.getSecondsFromPrevTime(newPrevTime, true);
    }
    return {
        ...Time.getTimeFromSeconds(seconds.value),
        start,
        pause,
        reset,
        isRunning,
    };
}

function useTime({ format }) {
    const seconds = vue.ref(Time.getSecondsFromTimeNow());
    useInterval(() => {
        seconds.value = Time.getSecondsFromTimeNow();
    }, 1000);
    return {
        ...Time.getFormattedTimeFromSeconds(seconds.value, format),
    };
}

exports.useStopwatch = useStopwatch;
exports.useTime = useTime;
exports.useTimer = useTimer;
